#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_live_tools_routing_table_lookups_info
short_description: Information module for devices _live _tools _routing _table _lookups
description:
  - Get devices _live _tools _routing _table _lookups by id. - > Return a routing
    table live tool lookup job for a device. The response contains the details of
    the job, including the filters used for the routing table lookup request and the
    results of the request if the job has completed. Only Cisco Secure Routers are
    supported.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  serial:
    description:
      - Serial path parameter.
    type: str
  id:
    description:
      - Id path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices getDeviceLiveToolsRoutingTableLookup
    description: Complete reference of the getDeviceLiveToolsRoutingTableLookup API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-device-live-tools-routing-table-lookup
notes:
  - SDK Method used are
    devices.Devices.get_device_live_tools_routing_table_lookup,
  - Paths used are
    get /devices/{serial}/liveTools/routingTable/lookups/{id},
"""

EXAMPLES = r"""
- name: Get devices _live _tools _routing _table _lookups by id
  cisco.meraki.devices_live_tools_routing_table_lookups_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serial: string
    id: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "lookupId": "string",
      "url": "string",
      "request": {
        "serial": "string",
        "type": "string",
        "destination": {
          "address": "string",
          "subnet": "string"
        },
        "nextHop": {
          "address": "string"
        },
        "vpn": {
          "peer": {
            "id": "string"
          }
        }
      },
      "status": "string",
      "entries": [
        {
          "type": "string",
          "subnet": "string",
          "nextHops": [
            {
              "number": 0,
              "address": "string",
              "vpn": {
                "peer": {
                  "id": "string",
                  "name": "string"
                }
              }
            }
          ],
          "ipVersion": "string",
          "vrf": {
            "name": "string"
          }
        }
      ],
      "errors": [
        "string"
      ]
    }
"""
